@@ -1,4 +1,4 @@
-# $Id: crt.py 924 2014-06-03 11:48:08Z mwall $
+# $Id: crt.py 926 2014-06-05 08:54:32Z mwall $
 # Copyright 2013 Matthew Wall
 
 # Emit loop data to file in Cumulus realtime format.
@@ -32,6 +32,7 @@
 
 import math
 import time
+import syslog
 
 import weewx
 import weewx.wxformulas
@@ -110,6 +111,8 @@ def calcAvgWindSpeed(archive, ts, interval=600):
     val = archive.getSql("SELECT AVG(windSpeed) FROM archive "
                          "WHERE dateTime>? AND dateTime<=?",
                          (sts, ts))
+    if val is None:
+        return None
     return val[0]
 
 def calcAvgWindDir(archive, ts, interval=600):
@@ -117,6 +120,8 @@ def calcAvgWindDir(archive, ts, interval=600):
     val = archive.getSql("SELECT AVG(windDir) FROM archive "
                          "WHERE dateTime>? AND dateTime<=?",
                          (sts, ts))
+    if val is None:
+        return None
     return val[0]
 
 def calc10MinHighGust(archive, ts):
@@ -124,6 +129,8 @@ def calc10MinHighGust(archive, ts):
     val = archive.getSql("SELECT MAX(windGust) FROM archive "
                          "WHERE dateTime>? AND dateTime<=?",
                          (sts, ts))
+    if val is None:
+        return None
     return val[0]
 
 def calc10MinAvgWindDir(archive, ts):
@@ -131,6 +138,8 @@ def calc10MinAvgWindDir(archive, ts):
     val = archive.getSql("SELECT AVG(windGust) FROM archive "
                          "WHERE dateTime>? AND dateTime<=?",
                          (sts, ts))
+    if val is None:
+        return None
     return val[0]
 
 # calculate wind run since midnight
@@ -150,6 +159,8 @@ def calcTrend(label, new_val, archive, ts, n=3):
     lastts = ts - n * 3600
     qstr = "SELECT %s FROM archive WHERE dateTime>? AND dateTime<=?" % label
     old_val = archive.getSql(qstr, (lastts, ts))
+    if old_val is None:
+        return None
     return new_val - old_val[0]
 
 def calcRainHour(archive, ts):
@@ -157,6 +168,8 @@ def calcRainHour(archive, ts):
     val = archive.getSql("SELECT SUM(rain) FROM archive "
                          "WHERE dateTime>? AND dateTime<=?",
                          (sts, ts))
+    if val is None:
+        return None
     return val[0]
 
 def calcRainMonth(archive, ts):
@@ -164,6 +177,8 @@ def calcRainMonth(archive, ts):
     val = archive.getSql("SELECT SUM(rain) FROM archive "
                          "WHERE dateTime>? AND dateTime<=?",
                          (span.start, ts))
+    if val is None:
+        return None
     return val[0]
 
 def calcRainYear(archive, ts):
@@ -171,6 +186,8 @@ def calcRainYear(archive, ts):
     val = archive.getSql("SELECT SUM(rain) FROM archive "
                          "WHERE dateTime>? AND dateTime<=?",
                          (span.start, ts))
+    if val is None:
+        return None
     return val[0]
 
 def calcRainYesterday(archive, ts):
@@ -179,6 +196,8 @@ def calcRainYesterday(archive, ts):
     val = archive.getSql("SELECT SUM(rain) FROM archive "
                          "WHERE dateTime>? AND dateTime<=?",
                          (sts, ts))
+    if val is None:
+        return None
     return val[0]
 
 def calcETToday(archive, ts):
@@ -186,14 +205,20 @@ def calcETToday(archive, ts):
     val = archive.getSql("SELECT SUM(ET) FROM archive "
                          "WHERE dateTime>? AND dateTime<=?",
                          (sts, ts))
+    if val is None:
+        return None
     return val[0]
 
 def calcMinMax(label, archive, ts, minmax='MAX'):
     sts = weeutil.weeutil.startOfDay(ts)
     qstr = "SELECT %s(%s) FROM archive WHERE dateTime>=? AND dateTime<=?" % (minmax,label)
     val = archive.getSql(qstr, (sts, ts))
+    if val is None:
+        return None, None
     qstr = "SELECT dateTime FROM archive WHERE dateTime>=? AND dateTime<=? AND %s=?" % label
     t = archive.getSql(qstr, (sts, ts, val[0]))
+    if t is None:
+        return None, None
     tstr = time.strftime("%H:%M", time.gmtime(t[0]))
     return val[0],tstr
 
@@ -277,16 +302,19 @@ class CumulusRealTime(StdService):
         self.latitude = getLatitude(config_dict)
         self.longitude = getLongitude(config_dict)
 
-        d = config_dict.get('CumulusRealTimeService', {})
+        d = config_dict.get('CumulusRealTime', {})
         self.filename = d.get('filename', '/var/tmp/realtime.txt')
         self.datesep = d.get('date_separator', '/')
         self.bind(weewx.NEW_LOOP_PACKET, self.handle_new_loop)
 
     def handle_new_loop(self, event):
-        data = {}
-        with weewx.archive.Archive.open(self.database_dict) as archive:
-            data = self.calculate(event.packet, archive)
-        self.write_data(data)
+        try:
+            data = {}
+            with weewx.archive.Archive.open(self.database_dict) as archive:
+                data = self.calculate(event.packet, archive)
+            self.write_data(data)
+        except Exception, e:
+            weeutil.weeutil.log_traceback('crt: **** ')
 
     def write_data(self, data):
         with open(self.filename, 'w') as f:
@@ -314,9 +342,11 @@ class CumulusRealTime(StdService):
         ut = weewx.units.getStandardUnitType(pu, 'rain')
         data['units_rain'] = UNITS_RAIN.get(ut[0],ut[0])
         data['wind_run'] = calcWindRun(archive, ts)
-        data['pressure_trend'] = calcTrend('barometer', packet.get('barometer'),
+        data['pressure_trend'] = calcTrend('barometer',
+                                           packet.get('barometer'),
                                            archive, ts)
-        data['temperature_trend'] = calcTrend('outTemp', packet.get('outTemp'),
+        data['temperature_trend'] = calcTrend('outTemp',
+                                              packet.get('outTemp'),
                                               archive, ts)
         data['rain_month'] = calcRainMonth(archive, ts)
         data['rain_year'] = calcRainYear(archive, ts)
@@ -352,7 +382,7 @@ class CumulusRealTime(StdService):
         t_C = weewx.units.convert(vt, 'degree_C')[0]
         vt = (packet.get('dewpoint'), ut[0], 'group_temperature')
         dp_C = weewx.units.convert(vt, 'degree_C')[0]
-        dp_K = dp_C + 273.15
+        dp_K = dp_C + 273.15 if dp_C is not None else None
         v = calcHumidex(t_C, dp_K);
         vt = (v, 'degree_C', 'group_temperature')
         v = weewx.units.convert(vt, ut[0])[0]
